// simple C interface to a PASM driver by Jon MacPhalen
// by David Betz
// added RGBW by Darren Payson

#include <propeller.h>
#include "rgbx.h"

#ifndef TRUE
#define TRUE    1
#define FALSE   0
#endif

// driver header structure
typedef struct {
    uint32_t    jmp_inst;
    uint32_t    resettix;
    uint32_t    bit0hi;
    uint32_t    bit0lo;
    uint32_t    bit1hi;
    uint32_t    bit1lo;
    uint32_t    swaprg;
    uint32_t    pixelbits;
} rgbx_hdr;

int sk6812_init(rgbx_t *state)
{
    return rgbx_init(state, 80, 300, 900, 600, 600, TYPE_GRB, 24);
}
int sk6812RGBW_init(rgbx_t *state)
{
    return rgbx_init(state, 80, 300, 900, 600, 600, TYPE_GRB, 32);
}

int ws2811_init(rgbx_t *state)
{
    return rgbx_init(state, 50, 250, 1000, 600, 650, TYPE_RGB, 24);
}

int ws2812_init(rgbx_t *state)
{
    return rgbx_init(state, 50, 400, 800, 700, 600, TYPE_GRB, 24);
}

int ws2812b_init(rgbx_t *state)
{
    return rgbx_init(state, 50, 350, 850, 800, 450, TYPE_GRB, 24);
}

// -- usreset is reset timing (us)
// -- ns0h is 0-bit high timing (ns)
// -- ns0l is 0-bit low timing (ns)
// -- ns1h is 1-bit high timing (ns)
// -- ns1l is 1-bit low timing (ns)
// -- type is TYPE_GRB for ws2812 and above
// -- 24 bit or 32 bit strips
int rgbx_init(rgbx_t *state, int usreset, int ns0h, int ns0l, int ns1h, int ns1l, int type, int bits)
{
    extern uint32_t binary_rgbx_driver_dat_start[];
    rgbx_hdr *hdr = (rgbx_hdr *)binary_rgbx_driver_dat_start;
    uint32_t ustix;

    ustix = CLKFREQ / 1000000;          // ticks in 1us

    hdr->resettix = ustix * usreset;
    hdr->bit0hi   = ustix * ns0h / 1000;
    hdr->bit0lo   = ustix * ns0l / 1000;
    hdr->bit1hi   = ustix * ns1h / 1000;
    hdr->bit1lo   = ustix * ns1l / 1000;
    hdr->swaprg   = (type == TYPE_GRB);
    hdr->pixelbits = bits;
    
    state->command = 0;
    state->cog = cognew(hdr, &state->command);
    
    return state->cog;
}

void rgbx_close(rgbx_t *state)
{
    if (state->cog >= 0) {
        cogstop(state->cog);
        state->cog = -1;
    }
}

void rgbx_refresh(rgbx_t *state, int pin, uint32_t *colors, int count)
{
    uint32_t cmd;
    cmd =  pin
        | ((count - 1) << 9)
        | ((uint32_t)colors << 16);
    while (state->command)
        ;
    state->command = cmd;
}

uint32_t rgbx_wheel(int pos)
{
    uint32_t color;

// Creates color from 0 to 255 position input
// -- colors transition r-g-b back to r

    // red range
    if (pos < 85)
        color = COLOR(255-pos*3, pos*3, 0, 0);
    
    // green range
    else if (pos < 170) {
        pos -= 85;
        color = COLOR(0, 255-pos*3, pos*3, 0);
    }
    
    // blue range
    else {
        pos -= 170;
        color = COLOR(pos*3, 0, 255-pos*3, 0);
    }
    
    return color;
}

uint32_t rgbx_wheel_dim(int pos, int brightness)
{
    uint32_t color;

// Creates color from 0 to 255 position input
// -- colors transition r-g-b back to r

    // red range
    if (pos < 85)
        color = COLORX(255-pos*3, pos*3, 0, 0,  brightness);
    
    // green range
    else if (pos < 170) {
        pos -= 85;
        color = COLORX(0, 255-pos*3, pos*3, 0,  brightness);
    }
    
    // blue range
    else {
        pos -= 170;
        color = COLORX(pos*3, 0, 255-pos*3, 0,  brightness);
    }
    
    return color;
}

/*

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/
